

class QueueMessage:
    def __init__(self, source, destination, action, message):
        self.source = source
        self.destination = destination
        self.message = message,
        self.action = action

    def __repr__(self):
        return f"source:{self.source} destination:{self.destination} message:{self.message}"


import asyncio
import signal
import os
import urllib3
from logger_service import Logger
from msb_handler import microServiceBusHandler
from base_service import BaseService, CustomService
from vpn_helper import VPNHelper


class Orchestrator(BaseService):
    def __init__(self, id, queue):

        self.run = True
        self.services = [self]
        self.queue = queue
        self.loop = asyncio.get_event_loop()
        urllib3.disable_warnings()

        super(Orchestrator, self).__init__(id, queue)

    async def Start(self):
        # signals are not implemented in windows :(
        if os.name != 'nt':
            signals = (signal.SIGTERM, signal.SIGINT)
            for s in signals:
                self.loop.add_signal_handler(
                    s, lambda s=s: asyncio.create_task(self.shutdown(s, self.loop)))

        logger = Logger("logger", self.queue)
        await self.StartService(logger)
        msbHandler = microServiceBusHandler("msb", self.queue)
        await self.StartService(msbHandler)
        vpnHelper = VPNHelper("vpnhelper", self.queue)
        await self.StartService(vpnHelper)

        # region
        text = """\033[92m
           _               ____                  _          ____             
 _ __ ___ (_) ___ _ __ ___/ ___|  ___ _ ____   _(_) ___ ___| __ ) _   _ ___  
| '_ ` _ \\| |/ __| '__/ _ \\___ \\ / _ \\ '__\\ \\ / / |/ __/ _ \\  _ \\| | | / __| 
| | | | | | | (__| | | (_) |__) |  __/ |   \\ V /| | (_|  __/ |_) | |_| \\__ \\ 
|_| |_| |_|_|\\___|_|  \\___/____/ \\___|_|    \\_/ |_|\\___\\___|____/ \\__,_|___/ \n\033[1mAXIANS IoT Operations - Python IoT agent\nfor more information visit https://microservicebus.com\n\033[0m"""
        self.printf(text)
        # endregion

        await self.Debug("Started")

        while self.run:
            msg = await self.queue.get()

            if msg.destination == "*":  # Send to all services
                for service in self.services:
                    try:
                        function = getattr(service, msg.action)
                        self.loop.create_task(function(msg))
                    except:
                        await self.Debug(f"{service.id} has no function called {msg.action}")
            else:  # Send to one service
                service = next(
                    (srv for srv in self.services if srv.id == msg.destination), None)
                if service != None:
                    function = getattr(service, msg.action)
                    self.loop.create_task(function(msg))
                else:
                    await self.Debug(f"There is no service called {msg.destination}")

            await asyncio.sleep(0.1)

    async def _start_service(self, message):
        service = message.message[0]
        self.services.append(service)
        task = self.loop.create_task(service.Start())
        service.task = task
        task.set_name(f"{task.get_name()} : {service.id}")
        task.add_done_callback(self.service_completed)
        await self.Debug(f"Running {len(self.services)} services")

    async def _stop_custom_services(self, message):
        # First stop all custom services
        customServices = [
            service for service in self.services if isinstance(service, CustomService)]

        for srv in customServices:
            srv.task.cancel()
            
        [await service.Stop() for service in customServices]
        [self.services.remove(service) for service in customServices]
        customServices = [
            service for service in self.services if isinstance(service, CustomService)]
        await self.Debug(f"Running {len(self.services)} services")

    def service_completed(self, fn):
        self.printf("*************************")
        self.printf(f"{fn.get_name()} stopped")
        self.printf("*************************")

    async def shutdown(self, signal, loop, *args):
        self.printf("")
        self.printf(f"Shutting down ...")
        self.printf(f"Running {len(self.services)} services")
        self.run = False
        
        # Calling all services to shutdown
        [await service.Stop() for service in self.services]

        # Terminating all tasks
        tasks = [t for t in asyncio.all_tasks() if t is not
                 asyncio.current_task()]

        [task.cancel() for task in tasks]

        await asyncio.gather(*tasks, return_exceptions=True)
        loop.stop()




import subprocess, sys, importlib, socket, array, struct, fcntl 

def get_public_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    ip = s.getsockname()[0]
    s.close()
    return ip

def getHwAddr(ifname):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    info = fcntl.ioctl(s.fileno(), 0x8927,  struct.pack('256s', bytes(ifname, 'utf-8')[:15]))
    return ':'.join('%02x' % b for b in info[18:24])



from rauc_handler import RaucHandler
from base_service import BaseService
from pathlib import Path
from signalrcore.hub_connection_builder import HubConnectionBuilder
from packaging import version
import time
import asyncio
import importlib
import uuid
import re
import os
import sys
import traceback
import socket
import requests
import os
import json
import psutil
import time
import logging
import glob
import urllib.request
import threading
import platform
import utils
import ssl
ssl._create_default_https_context = ssl._create_unverified_context


class microServiceBusHandler(BaseService):
    # region Constructor
    def __init__(self, id, queue):
        self.ready = False
        self.base_uri = "https://microservicebus.com"
        home = str(Path.home())
        self.msb_dir = f"{os.environ['HOME']}/msb-py"

        self.printf(f"os.environ =  {os.environ}")

        self.service_path = f"{self.msb_dir}/services"
        self.msb_settings_path = f"{self.msb_dir}/settings.json"
        self.rauc_handler = RaucHandler()
        super(microServiceBusHandler, self).__init__(id, queue)
    # endregion
    # region Base functions

    async def Start(self):
        try:
            await self.set_up_signalr()
            self.settings = self.get_settings()

            # If no sas key, try provision using mac address
            sas_exists = "sas" in self.settings
            if(sas_exists == False):
                await self.Debug("Create node using mac address")

                await self.create_node()

            else:
                await self.sign_in(self.settings, False)

            while True:
                await asyncio.sleep(0.1)
        except Exception as e:
            await self.ThrowError(f"Error in msb.start: {e}")
            await self.ThrowError(traceback.format_exc())

    async def _debug(self, message):
        pass

    # endregion
    # region Private methods
    def debug_sync(self, message):
        asyncio.run(self.Debug(message))

    def get_settings(self):
        settings = {
            "hubUri": self.base_uri
        }
        # Check if directory exists
        if os.path.isdir(self.msb_dir) == False:
            os.mkdir(self.msb_dir)

        # Load settings file if exists
        self.printf(self.msb_settings_path)
        if os.path.exists(self.msb_settings_path):
            self.printf("Settings exists")
            with open(self.msb_settings_path) as f:
                settings = json.load(f)
        else:
            self.printf("Creating settings")
            settings = {
                "hubUri": self.base_uri
            }
            self.save_settings(settings)
        return settings

    def save_settings(self, settings):
        with open(self.msb_settings_path, 'w') as settings_file:
            json.dump(settings, settings_file)

    async def sign_in(self, settings, first_sign_in):
        if(first_sign_in == True):
            self.printf("Node created successfully")
            self.settings = settings
            self.save_settings(settings)

        await self.Debug("Signing in")
        await self.Debug("Node id: \033[95m" + settings["nodeName"] + "\033[0m")
        await self.Debug("Organization id:\033[95m " + settings["organizationId"] + "\033[0m")
        await self.Debug("Mac addresses:\033[95m " + ':'.join(re.findall('..', '%012x' % uuid.getnode())) + "\033[0m")

        hostData = {
            "id": "",
            "connectionId": "",
            "Name": settings["nodeName"],
            "machineName": "",
            "OrganizationID": settings["organizationId"],
            "npmVersion": "3.12.3",
            "sas": settings["sas"],
            "recoveredSignIn": "False",
            # "ipAddresses": "",
            "macAddresses": [':'.join(re.findall('..', '%012x' % uuid.getnode()))]
        }

        # Use for debugging
        self.connection.send("signIn", [hostData])
        #self.connection.send("signInAsync", [hostData])

    def sign_in_sync(self, settings, first_sign_in):
        asyncio.run(self.sign_in(settings, first_sign_in))

    async def create_node(self):
        mac = ':'.join(re.findall('..', '%012x' % uuid.getnode()))
        await self.Debug(mac)
        self.connection.send("createNodeFromMacAddress", [mac])

    def start_azure_iot_service(self, sign_in_response):
        try:
            state = sign_in_response["state"]
            asyncio.run(self.Debug(f"Device state is {state}"))
            if state == "InActive":
                return

            file_name = "pythonAzureIoTProvider.py"
            uri = f"{self.base_uri}/api/Scripts/00000000-0000-0000-0000-000000000001/{file_name}"
            service_file = requests.get(uri, allow_redirects=True)
            service_file_name = os.path.join(self.service_path, file_name)

            file = open(service_file_name, 'wb+')
            file.write(service_file.content)
            file.close()

            # pythonActivity["userData"]["type"].replace("_py", "")
            module_name = "AzureIoTService"
            # pythonActivity["userData"]["type"]
            service_name = "AzureIoTService"

            spec = importlib.util.spec_from_file_location(
                module_name, service_file_name)
            module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(module)
            MicroService = getattr(module, module_name)
            microService = MicroService(service_name.lower(), self.queue, sign_in_response)  # (id, queue, config)
            asyncio.run(self.StartService(microService))
            asyncio.run(self.Debug(f"Loading module {module_name}"))

        except Exception as e:
            asyncio.run(self.ThrowError(
                f"Error in msb.start_azure_iot_service: {e}"))

    async def refresh_vpn_settings(self, args):
        self.connection.send("getVpnSettings", [""])

    async def update_vpn_endpoint(self, args):
        self.connection.send("updateVpnEndpoint", [args.message[0]["ip"]])

    # endregion
    # region SignalR event listeners

    async def set_up_signalr(self):
        self.handler = logging.StreamHandler()
        self.handler.setLevel(logging.DEBUG)
        # Settings
        signalr_uri = f"{self.base_uri}/nodeHub"
        self.connection = HubConnectionBuilder()\
            .with_url(signalr_uri, options={"verify_ssl": False}) \
            .with_automatic_reconnect({
                "type": "interval",
                "keep_alive_interval": 10,
                "intervals": [1, 3, 5, 6, 7, 87, 3]
            }).build()

        self.connection.keepAliveIntervalInMilliseconds = 1000 * 60 * 3
        self.connection.serverTimeoutInMilliseconds = 1000 * 60 * 6
        # Default listeners
        self.connection.on_open(lambda: self.debug_sync(
            "connection opened and handshake received ready to send messages"))
        self.connection.on_close(lambda: self.debug_sync("connection closed"))
        self.connection.on_error(lambda data: self.debug_sync(
            f"An exception was thrown closed{data.error}"))

        # mSB.com listeners
        self.connection.on(
            "nodeCreated", lambda sign_in_info: self.sign_in_sync(sign_in_info[0], True))
        self.connection.on(
            "signInMessage", lambda sign_in_response: self.successful_sign_in(sign_in_response[0]))
        self.connection.on(
            "ping", lambda conn_id: self.ping_response(conn_id[0]))
        self.connection.on("errorMessage", lambda arg: self.debug_sync(arg[0]))
        self.connection.on("restart", lambda args: os.execv(
            sys.executable, ['python'] + sys.argv))
        self.connection.on("reboot", lambda args: os.system("/sbin/reboot"))
        self.connection.on("reset", lambda args: self.reset(args[0]))
        self.connection.on("heartBeat", lambda messageList: self.printf(
            "Heartbeat received: " + " ".join(messageList)))
        self.connection.on("reportState", lambda id: self.report_state(id[0]))
        self.connection.on("updateFirmware", lambda firmware_response: self.update_firmware(
            firmware_response[0], firmware_response[1]))
        self.connection.on("setBootPartition", lambda boot_info: self.set_boot_partition(
            boot_info[0], boot_info[1]))
        self.connection.on("getVpnSettingsResponse", lambda vpn_response: self.get_vpn_settings_response(
            vpn_response[0], vpn_response[1], vpn_response[2]))
        self.connection.on(
            "refreshVpnSettings", lambda response: self.refresh_vpn_settings_sync(response))
        self.connection.on(
            "sendMessage", lambda args: self.debug_sync(args[0]))

        # region Not implemented event handlers
        self.connection.on(
            "getEndpoints", lambda response: self.not_implemented("getEndpoints"))
        self.connection.on(
            "updateItinerary", lambda response: self.not_implemented("updateItinerary"))
        self.connection.on(
            "changeState", lambda response: self.not_implemented("changeState"))
        self.connection.on(
            "changeDebug", lambda response: self.not_implemented("changeDebug"))
        self.connection.on(
            "changeTracking", lambda response: self.not_implemented("changeTracking"))
        #self.connection.on("sendMessage", lambda response: self.not_implemented("sendMessage"))
        self.connection.on(
            "forceUpdate", lambda response: self.not_implemented("forceUpdate"))
        self.connection.on(
            "restartCom", lambda response: self.not_implemented("restartCom"))
        self.connection.on(
            "shutdown", lambda response: self.not_implemented("shutdown"))
        self.connection.on(
            "refreshSnap", lambda response: self.not_implemented("refreshSnap"))
        self.connection.on("resetKeepEnvironment", lambda response: self.not_implemented(
            "resetKeepEnvironment"))
        self.connection.on(
            "updateFlowState", lambda response: self.not_implemented("updateFlowState"))
        self.connection.on(
            "enableDebug", lambda response: self.not_implemented("enableDebug"))
        self.connection.on(
            "uploadSyslogs", lambda response: self.not_implemented("uploadSyslogs"))
        self.connection.on(
            "resendHistory", lambda response: self.not_implemented("resendHistory"))
        self.connection.on(
            "requestHistory", lambda response: self.not_implemented("requestHistory"))
        self.connection.on(
            "transferToPrivate", lambda response: self.not_implemented("transferToPrivate"))
        self.connection.on(
            "grantAccess", lambda response: self.not_implemented("grantAccess"))
        self.connection.on(
            "runTest", lambda response: self.not_implemented("runTest"))
        self.connection.on(
            "pingNodeTest", lambda response: self.not_implemented("pingNodeTest"))
        self.connection.on(
            "updatePolicies", lambda response: self.not_implemented("updatePolicies"))
        self.connection.on(
            "executeScript", lambda response: self.not_implemented("executeScript"))
        self.connection.on("updateVulnerabilities", lambda response: self.not_implemented(
            "updateVulnerabilities"))
        self.connection.on(
            "dockerListImages", lambda response: self.not_implemented("dockerListImages"))
        self.connection.on("dockerListContainers", lambda response: self.not_implemented(
            "dockerListContainers"))
        self.connection.on("dockerInstallImage", lambda response: self.not_implemented(
            "dockerInstallImage"))
        self.connection.on(
            "dockerDeleteImage", lambda response: self.not_implemented("dockerDeleteImage"))
        self.connection.on("dockerStartContainer", lambda response: self.not_implemented(
            "dockerStartContainer"))
        self.connection.on("dockerStopContainer", lambda response: self.not_implemented(
            "dockerStopContainer"))
        self.connection.on(
            "dockerComposeList", lambda response: self.not_implemented("dockerComposeList"))
        self.connection.on("dockerComposeInstall", lambda response: self.not_implemented(
            "dockerComposeInstall"))
        self.connection.on(
            "dockerComposeUp", lambda response: self.not_implemented("dockerComposeUp"))
        self.connection.on(
            "dockerComposeDown", lambda response: self.not_implemented("dockerComposeDown"))
        self.connection.on(
            "startTerminal", lambda response: self.not_implemented("startTerminal"))
        self.connection.on(
            "stopTerminal", lambda response: self.not_implemented("stopTerminal"))
        self.connection.on(
            "terminalCommand", lambda response: self.not_implemented("terminalCommand"))
        self.connection.on(
            "downloadFile", lambda response: self.not_implemented("downloadFile"))
        self.connection.on(
            "uploadFile", lambda response: self.not_implemented("uploadFile"))
        # endregion

        self.connection.start()
        time.sleep(1)
        self.set_interval(self.sendHeartbeat, 60 * 3)
    # endregion
    # region SignalR callback functions

    def successful_sign_in(self, sign_in_response):
        node_name = sign_in_response["nodeName"]
        tag_list = sign_in_response["tags"]

        asyncio.run(self.Debug(f"Node {node_name} signed in successfully"))
        self.save_settings(sign_in_response)

        asyncio.run(self.SubmitAction("*", "msb_signed_in", {}))

        if sign_in_response['protocol'] == "AZUREIOT":
            self.start_azure_iot_service(sign_in_response)

        if os.path.isdir(self.service_path) == False:
            os.mkdir(self.service_path)

        if sign_in_response['itineraries'] is not None:
            for itinerary in sign_in_response['itineraries']:
                pythonActivities = [srv for srv in itinerary["activities"]
                                    if srv["userData"]["baseType"] == 'pythonfile']
                for pythonActivity in pythonActivities:

                    host_config = [srv for srv in pythonActivity["userData"]
                                   ["config"]["generalConfig"] if srv["id"] == 'host']

                    # Check if activity is set to run on node
                    if host_config[0]["value"] != node_name and host_config[0]["value"] not in tag_list:
                        continue

                    organization_id = sign_in_response["organizationId"]
                    file_name = pythonActivity["userData"]["type"].replace(
                        "_py", ".py")

                    uri = f"{self.base_uri}/api/Scripts/{organization_id}/{file_name}" if pythonActivity["userData"][
                        "isCustom"] == True else f"{self.base_uri}/api/Scripts/00000000-0000-0000-0000-000000000001/{file_name}"

                    service_file = requests.get(uri, allow_redirects=True)
                    service_file_name = os.path.join(
                        self.service_path, file_name)

                    file = open(service_file_name, 'wb+')
                    file.write(service_file.content)
                    file.close()

                    module_name = pythonActivity["userData"]["type"].replace(
                        "_py", "")
                    service_name = pythonActivity["userData"]["type"]
                    service_config = pythonActivity["userData"]["config"]

                    spec = importlib.util.spec_from_file_location(
                        module_name, service_file_name)
                    module = importlib.util.module_from_spec(spec)
                    spec.loader.exec_module(module)
                    MicroService = getattr(module, module_name)
                    microService = MicroService(service_name.lower(
                    ), self.queue, service_config)  # (id, queue, config)
                    asyncio.run(self.StartService(microService))
                    asyncio.run(self.Debug(f"Loading module {module_name}"))

        self.sendHeartbeat()

    def not_implemented(self, event_handler):
        asyncio.run(self.ThrowError(
            f'SignalR event handler \033[93m"{event_handler}"\033[0m is not implemented in the Python Node'))

    def ping_response(self, conn_id):
        self.printf("Ping response")
        settings = self.get_settings()
        self.connection.send("pingResponse", [
                             settings["nodeName"], socket.gethostname(), "Online", conn_id, False])

    def sendHeartbeat(self):
        self.connection.send("heartBeat", ["echo"])

    def report_state(self, id):
        node_name = self.settings["nodeName"]
        self.printf(f'Fetching environment state from {node_name}')
        self.connection.send(
            'notify', [id, f'Fetching environment state from {node_name}', 'INFO'])

        networks = []
        # interfaces = netifaces.interfaces()
        # for interface in interfaces:
        #     addrs = netifaces.ifaddresses(interface)
        #     if netifaces.AF_INET in addrs.keys():
        #         ni = {
        #                 "name": interface,
        #                 "ip_address":addrs[netifaces.AF_INET][0]["addr"],
        #                 "mac_address": utils.getHwAddr(interface),
        #                 "netmask":addrs[netifaces.AF_INET][0]["netmask"],
        #                 "type": ''}
        #         networks.append(ni)
        memory_info = psutil.virtual_memory()
        if_addrs = psutil.net_if_addrs()
        cpu_times = psutil.cpu_times()
        disk_info = psutil.disk_usage('/')
        slot_status = self.rauc_handler.get_slot_status()
        state = {
            "networks": networks,
            "memory": {
                "totalMem": f'{(memory_info.total / 1000 / 1000):9.2f} Mb',
                "freemem": f'{(memory_info.free / 1000 / 1000):9.2f} Mb'
            },
            "cpus": [{
                "model": platform.processor(),
                "speed": None,
                "times": {
                    "user": cpu_times.user,
                    "nice": cpu_times.nice,
                    "sys": cpu_times.system,
                    "idle": cpu_times.idle
                }
            }],
            "env": dict(os.environ),
            "storage": {
                "available": f'{(disk_info.total / 1000 / 1000):9.2f} Mb',
                "free": f'{(disk_info.free / 1000 / 1000):9.2f} Mb',
                "total": f'{(disk_info.total / 1000 / 1000):9.2f} Mb'
            },
            "raucState": slot_status
        }
        self.connection.send('reportStateResponse', [state, id])

    def reset(self, id):
        asyncio.run(self.Debug("\033[93mResetting node\033[0m"))
        node_name = self.settings["nodeName"]
        settings = {
            "hubUri": self.base_uri
        }
        self.save_settings(settings)
        self.connection.send(
            'notify', [id, f"Node {node_name} has been reset", "INFO"])
        os.execv(sys.executable, ['python'] + sys.argv)

    def update_firmware(self, force, connid):
        self.printf(force)
        self.printf(connid)
        platform_status = dict(self.rauc_handler.get_slot_status())
        print(platform_status)
        print(type(platform_status))
        rootfs0_status = platform_status["rootfs0"]["state"]
        current_platform = platform_status["rootfs0"] if rootfs0_status == "booted" else platform_status["rootfs1"]
        platform = current_platform["bundle.compatible"]
        current_version = current_platform["bundle.version"]
        boot_status = current_platform["boot-status"]
        installed = current_platform["installed.timestamp"]

        uri = "https://microservicebus.com/api/nodeimages/" + \
            self.get_settings()["organizationId"] + "/" + platform
        self.printf("Notified on new firmware")
        self.printf("Current firmware platform: " + platform)
        self.printf("Current firmware version: " + current_version)
        self.printf("Current boot status: " + boot_status)
        self.printf("Current firmware installed: " + installed)

        self.printf("Fetching meta data from: " + uri)

        response = requests.get(uri)
        if(response.status_code != 200):
            self.printf("No firmware image found")
            return
        metamodel = response.json()
        if(force or version.parse(metamodel["version"]) > version.parse(current_version)):
            self.printf("New firmware version found")
            dir = f"{self.msb_dir}/firmwareimages/"
            # Check if directory exists
            if os.path.isdir(dir) == False:
                os.mkdir(dir)
            files = glob.glob(dir + "*")
            for f in files:
                os.remove(f)
            self.printf("Files removed")
            file_name = os.path.join(dir, os.path.basename(metamodel["uri"]))
            self.printf(file_name)
            urllib.request.urlretrieve(metamodel["uri"], file_name)
            self.printf("Download complete")
            self.printf("Calling RAUC")
            self.printf(f"Installing {file_name}")

            self.rauc_handler.install(file_name)

    def set_boot_partition(self, partition, connid):
        self.printf(f"Marking partition {partition}")
        self.rauc_handler.mark_partition("active", partition)
        self.printf("Successfully marked partition")
        self.connection.send(
            "notify", [connid, f"Successfully marked partition.", "INFO"])
        time.sleep(10)
        os.system("sudo /sbin/reboot")

    def set_interval(self, func, sec):
        def func_wrapper():
            self.set_interval(func, sec)
            func()
        t = threading.Timer(sec, func_wrapper)
        t.start()
        return t

    def refresh_vpn_settings_sync(self, args):
        self.connection.send("getVpnSettings", [""])

    def get_vpn_settings_response(self, vpnConfig, interfaceName, endpoint):
        message = {'vpnConfig': vpnConfig,
                   'interfaceName': interfaceName,
                   'endpoint': endpoint,
                   'vpnConfigPath': f"{self.msb_dir}/{interfaceName}.conf"}

        asyncio.run(self.SubmitAction(
            "vpnhelper", "get_vpn_settings_response", message))

    # endregion



import sys
import subprocess
import asyncio
import utils
from xmlrpc.client import Server
from base_service import BaseService

class VPNHelper(BaseService):
    def __init__(self, id, queue):
        super(VPNHelper, self).__init__(id, queue)

    async def Start(self):
        await self.Debug("Started")
        while True:
            await asyncio.sleep(0.1)

    async def msb_signed_in(self, args):
        try:
            # Check if wireguard is installed
            response = subprocess.run("wg --help", stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, shell=True)
            if(response.returncode != 0):
                return

            message = [1, 2, 3]
            await self.SubmitAction("msb", "refresh_vpn_settings", message[0])

        except subprocess.CalledProcessError as e:
            self.printf(e.output)

    async def get_vpn_settings_response(self, args):
        try:
            await self.Debug("Received VPN settings from cloud")
            message = args.message[0]
            if message["vpnConfigPath"]:
                self.vpnConfigPath = message["vpnConfigPath"];           
                ip = utils.get_public_ip()
                
                if ip != message["endpoint"]:
                    await self.Debug(f"IP has changed ({ip}). Calling cloud to update")
                    req = {'ip': ip}
                    await self.SubmitAction("msb", "update_vpn_endpoint", req)

                wg_config_file = open(message["vpnConfigPath"], "w")
                wg_config_file.write(message["vpnConfig"])
                wg_config_file.close()
                
                await self.Debug("wireguard config has been updated")
                await self.wg_down()
                await self.wg_up()
            else:
                # server.config().post_down
                await self.Debug("down")

        except Exception as e:
            await self.Debug(f"Error in vpn_helper down: {e}")
    
    async def wg_up(self):
        try:
            wg_command =  f"wg-quick up {self.vpnConfigPath}"
            response = subprocess.run( wg_command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, shell=True)
            if(response.returncode != 0):
                await self.Debug("Failed to bring up VPN connection")
            else:
                await self.Debug("VPN connection established")
        except Exception as e:
            await self.Debug(f"Error trying to connect VPN: {e}")
            
    async def wg_down(self):
        try:
            wg_command =  f"wg-quick down {self.vpnConfigPath}"
            response = subprocess.run( wg_command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, shell=True)
            if(response.returncode != 0):
                await self.Debug(f"Failed to bring down VPN connection. This is probobly ok")
            else:
                await self.Debug("VPN connection disconnected")
        except Exception as e:
            await self.Debug(f"Error trying to disconnect VPN: {e}")





import asyncio, logging
from base_service import BaseService

class AzureIoT(BaseService):
    def __init__(self, id, queue, provider_info):
        logger = logging.getLogger('azure.iot.device')
        logger.setLevel(logging.WARNING)
        
        try:
            IoTHubDeviceClient = self.AddPipPackage("azure-iot-device", "azure.iot.device.aio", "IoTHubDeviceClient")
        except Exception as e:
            self.printf(f"Error installing azure-iot-device: {e}")

        self.device_client = IoTHubDeviceClient.create_from_connection_string(provider_info["connectionString"])
        self.device_client.on_method_request_received = self.method_request_handler
        self.device_client.on_twin_desired_properties_patch_received = self.twin_patch_handler
        self.twin = None
        super(AzureIoT, self).__init__(id, queue)

    async def Start(self):
        if self.device_client == None:
            await self.Debug("azure-iot-device is not installed")
            return

        await self.device_client.connect()
        self.twin = await self.device_client.get_twin()
        await self.SubmitAction("*", "StateUpdate", self.twin)

        state_exists = "msb-state" in self.twin["desired"]
        if state_exists:
            if(self.twin["desired"]["msb-state"]["enabled"] == False):
                await self.StopCustomServices()
        else:
            await self.Debug("msb-state does not exist in twin")
                
        await self.Debug("Com is started")
        while True:
            await asyncio.sleep(0.1)

    async def Stop(self):
        await self.Debug("Disconnecting device_client")
        await self.device_client.disconnect()

    async def _sendEvent(self, message):
        if self.device_client.connected:
            await self.device_client.send_message(message.message[0])
            await self.Debug("Message sent to IoT Hub")
        else:
            await self.Debug("Unable to send message to IoT Hub. Reason: Not connected")
    
    async def twin_patch_handler(self, patch):
        
        local_state_exists = "msb-state" in self.twin["desired"]
        patch_state_exists = "msb-state" in patch

        if local_state_exists == False:
            self.twin["desired"] = patch

        if patch_state_exists:
            print("patch_state_exists")
            if(self.twin["desired"]["msb-state"]["enabled"] != patch["msb-state"]["enabled"]):
                if(patch["msb-state"]["enabled"] == False):
                    await self.StopCustomServices()
                else:
                    await self.StartCustomServices()
                    
        self.twin = await self.device_client.get_twin()
        await self.SubmitAction("*", "StateUpdate", self.twin)
            
    async def method_request_handler(self, method_request):
        await self.Debug(f"Inbound call: {method_request.name}")
        # Determine how to respond to the method request based on the method name
        if method_request.name == "stop":
            await self.StopCustomServices()
            payload = {"result":"All services stopped"}
            status = 200 
        elif method_request.name == "start":
            await self.StartCustomServices()
            payload = {"result":"All services started"}
            status = 200 
        else:
            payload = {"result": False, "data": "unknown method"}  # set response payload
            status = 400  # set return status code
            print("executed unknown method: " + method_request.name)

        # Send the response
        MethodResponse = self.AddPipPackage("azure-iot-device", "azure.iot.device.aio", "MethodResponse")
        method_response = MethodResponse.create_from_method_request(method_request, status, payload)
        await self.device_client.send_method_response(method_response)



import asyncio
from base_service import BaseService

class Logger(BaseService):
    def __init__(self, id, queue):
        self.debug = True
        super(Logger, self).__init__(id, queue)

    async def Start(self):
        await self.Debug("Started")
        while True:
            await asyncio.sleep(0.1)
    
    async def StateUpdate(self, message):
        state = message.message[0]
        await self.Debug(f"Received: {message}")

    async def _debug(self, message):
       self.printf(f"mSB:[{message.source}] {bcolors.OKGREEN}DEGUG:{bcolors.ENDC} {message.message[0]}")
       if self.debug:
           await self.SubmitAction("msb", "_debug", message.message[0])
    
    async def _error(self, message):
       self.printf(f"mSB:[{message.source}] {bcolors.FAIL}ERROR:{bcolors.ENDC} {message.message[0]}")
       if self.debug:
           await self.SubmitAction("msb", "_debug", message.message[0])

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKCYAN = '\033[96m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


import asyncio
import subprocess
import importlib
from queue_message import QueueMessage


class BaseService:
    def __init__(self, id, queue):
        self.id = id
        self.queue = queue
        self.task = None

    def printf(self, msg):
        print(msg, flush=True)
        
    # region Default functions
    async def Start(self):
        pass

    async def Stop(self):
        pass

    async def Process(self, message):
        pass

    async def StateUpdate(self, state):
        pass

    # endregion

    # region Communication functions
    async def SubmitAction(self, destination, action, message):
        msg = QueueMessage(self.id, destination, action, message)
        asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)

    async def Debug(self, message):
        msg = QueueMessage(self.id, "logger", "_debug", message)
        task = asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)

    async def ThrowError(self, message):
        msg = QueueMessage(self.id, "logger", "_error", message)
        task = asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)

    async def SubmitMessage(self, message):
        msg = QueueMessage(self.id, "com", "_sendEvent", message)
        asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)

    async def StartService(self, service):
        msg = QueueMessage(self.id, "orchestrator", "_start_service", service)
        asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)

    async def StopCustomServices(self):
        msg = QueueMessage(self.id, "orchestrator",
                           "_stop_custom_services", "")
        asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)

    async def StartCustomServices(self):
        msg = QueueMessage(self.id, "msb", "_start_custom_services", "")
        asyncio.create_task(self.queue.put(msg))
        await asyncio.sleep(0)
    # endregion

    # region Private functions
    # package = pip package E.g "azure-iot-device"
    # module = module name E.g "azure.iot.device.aio"
    # name = name of object E.g "IoTHubDeviceClient". If you don't know the name, try dir(importlib.import_module("module name")

    # SAMPLES:
    # IoTHubDeviceClient = self.AddPipPackage("azure-iot-device", "azure.iot.device.aio", "IoTHubDeviceClient")
    # MqttClient = self.AddPipPackage("paho-mqtt", "paho.mqtt.client", "Client")

    # Think of it this way.... MqttClient = self.AddPipPackage("paho-mqtt", "paho.mqtt.client", "Client") ...is equivalent to:
    # pip install paho-mqtt
    # from paho.mqtt.client import Client
    def AddPipPackage(self, package, module, name):
        try:
            try:
                self.printf(f"Importing {module}")
                importlib.import_module(module)
            except Exception as e1:
                self.printf (f"Unable to import module:. \nTrying to install package {package}")

                response = subprocess.run(f"pip3 install {package}", stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, shell=True)
                if(response.returncode != 0):
                    self.printf (f"Failed to install {package}")
                    raise Exception('InstallPackage', f"Failed to install {package}")
                else:
                     self.printf (f"Successfully installed {package}")
                    
        except Exception as e2:
            self.printf (f"Failed to install {package}")
            raise Exception('InstallPackage', f"Failed to install {package}")

        Package = getattr(importlib.import_module(module), name)
        return Package

    # endregion


class CustomService(BaseService):
     def __init__(self, id, queue, config):
        self.id = id
        self.queue = queue
        self.config = config
        super(CustomService, self).__init__(id, queue)
       
     def GetPropertyValue(self, category, prop):
         ## newDict = dict(filter(lambda elem: elem[0] % 2 == 0, dictOfNames.items()))
        try:
            category = category + "Config"
            values = [config for config in self.config[category] if config['id'] == prop]
            if len(values) > 0:
                return values[0]['value']
            else:
                return "undefined"
        except Exception as e:
                print(e, flush=True)
                return "undefined" 


#!/usr/bin/env python3.7
import asyncio
import logging
import os
import sys
sys.path.insert(1, os.path.dirname(__file__))
from orchestrator_service import Orchestrator

logging.basicConfig(
    level=logging.WARNING,
    format="%(asctime)s,%(msecs)d %(levelname)s: %(message)s",
    datefmt="%H:%M:%S",
)

def main():
    try:
        print(f"pid: {os.getpid()}")
        sys.path.append(os.path.abspath('../lib'))

        orchestrator = Orchestrator("orchestrator", asyncio.Queue())
        orchestrator.loop.create_task(orchestrator.Start())
        orchestrator.loop.run_forever()
    except KeyboardInterrupt:
        logging.info("Process interrupted")
    finally:
        orchestrator.loop.close()
        logging.info("Successfully shutdown the Mayhem service.")


if __name__ == "__main__":
    main()




import psutil, utils

networks = []
# interfaces = netifaces.interfaces()
# for interface in interfaces:
#     addrs = netifaces.ifaddresses(interface)
#     if netifaces.AF_INET in addrs.keys():
#         # print(interface)
#         # print(addrs[netifaces.AF_INET][0])
#         # print(utils.getHwAddr(interface))
#         ni = {
#                 "name": interface, 
#                 "ipaddress":addrs[netifaces.AF_INET][0]["addr"], 
#                 "mac_address": utils.getHwAddr(interface), 
#                 "netmask":addrs[netifaces.AF_INET][0]["netmask"],
#                 "type": ''}
#         networks.append(ni)

print (networks)




"""Setup script for realpython-reader"""

# Standard library imports
import os
import pathlib

# Third party imports
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).resolve().parent

# The text of the README file is used as a description
README = (HERE / "README.md").read_text()


thelibFolder = os.path.dirname(os.path.realpath(__file__))
requirementPath = thelibFolder + '/requirements.txt'
install_req = []  # Here we'll get: ["gunicorn", "docutils>=0.3", "lxml==0.5a7"]
if os.path.isfile(requirementPath):
    with open(requirementPath) as f:
        install_req = f.read().splitlines()

# This call to setup() does all the work
setup(
    name="microservicebus-py",
    version="0.0.15",
    description="Python agent for microServiceBus.com. Please visit https://microservicebus.com for more information.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/axians/microservicebus-py",
    author="AXIANS IoT Operations",
    author_email="microservicebus@axians.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=["src"],
    include_package_data=True,
    install_requires=install_req,
    entry_points={"console_scripts": ["microservicebus-py=src.start:main"]},
)



import signal
import os
import subprocess
import json
from functools import reduce
from gi.repository import GLib
from gi.repository import Gio


class RaucHandler():
    def __init__(self):
        try:
            self.bus = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
            self.dbus_proxy = Gio.DBusProxy.new_sync(self.bus, Gio.DBusProxyFlags.NONE,None,'de.pengutronix.rauc','/','de.pengutronix.rauc.Installer',None)
        except Exception as ex:
            self.printf("ups")

    def get_slot_status(self):
        response = subprocess.run("rauc status --detailed --output-format=json", stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, shell=True)
        if response.returncode != 0:
            return None
            
        result = json.loads(response.stdout)
        rootfs0 = next(item for item in result["slots"] if "rootfs.0" in item)["rootfs.0"]
        self.printf(rootfs0)
        rootfs1 = next(item for item in result["slots"] if "rootfs.1" in item)["rootfs.1"]
        slot_status = {
            "rootfs0": {
                "installed.count": self.deep_get(rootfs0, "slot_status.installed.count"),
                "status": self.deep_get(rootfs0, "slot_status.status"),
                "type": rootfs0["type"],
                "sha256": self.deep_get(rootfs0, "slot_status.checksum.sha256"),
                "bundle.compatible": self.deep_get(rootfs0, "slot_status.bundle.compatible"),
                "device": rootfs0["device"],
                "boot-status": self.deep_get(rootfs0, "slot_status.status"),
                "size": self.deep_get(rootfs0, "slot_status.checksum.size"),
                "bundle.build": self.deep_get(rootfs0, "slot_status.bundle.build"),
                "activated.timestamp": self.deep_get(rootfs0, "slot_status.activated.timestamp"),
                "installed.timestamp": self.deep_get(rootfs0, "slot_status.installed.timestamp"),
                "state": rootfs0["state"],
                "bootname": rootfs0["bootname"],
                "class": rootfs0["class"],
                "bundle.version": self.deep_get(rootfs0, "slot_status.bundle.version"),
                "bundle.description": self.deep_get(rootfs0, "slot_status.bundle.description"),
                "description": None,
                "activated.count": self.deep_get(rootfs0, "slot_status.activated.count")
            },
            "rootfs1": {
                "installed.count": self.deep_get(rootfs1, "slot_status.installed.count"),
                "status": self.deep_get(rootfs1, "slot_status.status"),
                "type": rootfs1["type"],
                "sha256": self.deep_get(rootfs1, "slot_status.checksum.sha256"),
                "bundle.compatible": self.deep_get(rootfs1, "slot_status.bundle.compatible"),
                "device": rootfs1["device"],
                "boot-status": self.deep_get(rootfs1, "slot_status.status"),
                "size": self.deep_get(rootfs1, "slot_status.checksum.size"),
                "bundle.build": self.deep_get(rootfs1, "slot_status.bundle.build"),
                "activated.timestamp": self.deep_get(rootfs1, "slot_status.activated.timestamp"),
                "installed.timestamp": self.deep_get(rootfs1, "slot_status.installed.timestamp"),
                "state": rootfs1["state"],
                "bootname": rootfs1["bootname"],
                "class": rootfs1["class"],
                "bundle.version": self.deep_get(rootfs1, "slot_status.bundle.version"),
                "bundle.description": self.deep_get(rootfs1, "slot_status.bundle.description"),
                "description": None,
                "activated.count": self.deep_get(rootfs1, "slot_status.activated.count")
            }
        }
        return slot_status

    def mark_partition(self, state, partition):
        mark_result = self.dbus_proxy.call_sync('Mark', GLib.Variant(
            '(ss)', (state, partition)), Gio.DBusCallFlags.NO_AUTO_START, 500, None)
        return mark_result

    def install(self, path):
        try:
            self.dbus_proxy.Install('(s)',(path))
            self.reboot_after_install()
        except Exception as e:
            self.printf(f"Error in rauc_handler.install: {e}")

    def reboot_after_install(self, nb):
        self.printf("Received completed from RAUC interface")
        os.system("/sbin/reboot")

    def deep_get(self, dictionary, keys, default=None):
        return reduce(lambda d, key: d.get(key, default) if isinstance(d, dict) else default, keys.split("."), dictionary)
